%differentiate
% Differentiation function using cubic splines.
%
% Syntax: dy_by_dx = differentiate( xx, yy, x )
%
% xx, yy     Samples from function y(x) which is being differentiated. These
%            must be the same dimensions and have at least three elements each.
% x          Point on y(x) at which the gradient is returned. This can be an array of any
%            dimensions.
%
% dy_by_dx   Output of gradient of y(x) corresponindg to array x
%
% The function is based on the code written on pages 292 and 294
% of "Mastering Matlab 6" by Duane Hanselman & Bruce Littlefield.
%
% LAST UPDATED by Andy French. July 2014.

function dy_by_dx = differentiate( xx, yy, x )

%Check inputs are the right size
if ( numel(xx) == numel(yy) ) && ( numel(xx) >= 3 )
    
    %Sort xx in ascending order
    [xx,i] = sort(xx);
    yy = yy(i);
    
    %Compute piecewise polynomial based upon cubic splines.
    pp=spline(xx,yy);
    %pp = pchip(xx,yy);  %Alternative Hermite polynomial fit. Apparently this
    %is slightly better at preserving the shaoe of the function near
    %discontinuities.
    
    %Take apart piecewise polynomial
    [br,co,npy,nco]=unmkpp(pp);
    
    %Scale factors for differentiation
    sf=nco-1:-1:1;
    
    %Derivative coefficients
    dco=sf(ones(npy,1),:).*co(:,1:nco-1);
    
    %Build pp form for derivative
    ppd=mkpp(br,dco); 
    
    %Generate the values of dy_by_dx by evaluating the piecewise polynomial
    dy_by_dx = ppval(ppd,x);
    
else
    error('x,y inputs are not the same size or are less than three elements.')
end

%%

function differentiate_test

x = linspace( -10, 10, 500 );
y = x.^2;
dy_by_dx = differentiate( x, y, x );
plot(x,y,x,dy_by_dx);
grid on
xlabel('x')
ylabel('y')

%End of code