%integrate
% Integration function using cubic splines.
%
% Syntax: I = integrate( xx, yy, x0, x )
%
% xx, yy     Samples from function y(x) which is being integrated. These
%            must be the same dimensions and have at least three elements each.
% x0         Starting point (i.e. lower limit) of the integral
% x          Upper limit of the integral. This can be an array of any
%            dimensions.
%
% I          Output of integral corresponindg to array x
%
% The function is based on the code written on pages 292 and 294
% of "Mastering Matlab 6" by Duane Hanselman & Bruce Littlefield.
%
% LAST UPDATED by Andy French. July 2014.

function I = integrate( xx, yy, x0, x )

%Check inputs are the right size
if ( numel(xx) == numel(yy) ) && ( numel(xx) >= 3 )
    
    %Sort xx in ascending order
    [xx,i] = sort(xx);
    yy = yy(i);
    
    %Compute piecewise polynomial based upon cubic splines.
    pp=spline(xx,yy);
    %pp = pchip(xx,yy);  %Alternative Hermite polynomial fit. Apparently this
    %is slightly better at preserving the shaoe of the function near
    %discontinuities.
    
    %Take apart piecewise polynomial
    [br,co,npy,nco] = unmkpp(pp);
    
    %Scale factors for integration
    sf=nco:-1:1;
    
    %Integral coefficients
    ico=[co./sf(ones(npy,1),:) zeros(npy,1)];
    
    %Integral spline has higher order
    nco=nco+1;
    
    %Set integration constant of all splines to be zero
    ico(1,nco)=0;
    
    %Find constant terms in polynomials
    for k=2:npy
        ico(k,nco)=polyval(ico(k-1,:),br(k)-br(k-1));
    end
    
    %Build pp form for integral
    ppi=mkpp(br,ico);
    
    %Evaluate definite integral between limits of x0 to x
    I = ppval(ppi,x) - ppval(ppi,x0);
    
else
    error('x,y inputs are not the same size or are less than three elements.')
end

%%

function integrate_test

x = linspace( -10, 10, 500 );
y = x;
I = integrate( x, y, -10, x );
plot(x,y,x,I);
grid on
xlabel('x')
ylabel('y')

%End of code