%plot_histogram_and_cum_freq
% Function which performs a statistical analysis of a set of sample inputs
% (or alternatively a frequency table) and then plots a histogram,
% a normalized histogram with overlaid fitted skew-normal distribution and
% a cumulative frequency graph.
%
% LAST UPDATED by Andy French July 2014
%
% Syntax: [ fig_hist, ax_hist, fig_norm, ax_norm, fig_cumf, ax_cumf, histogram_edges,f ] = ...
% plot_histogram_and_cum_freq( x_s, xmin, xmax, pdf_max, histogram_edges,f,...
% min_bin_edge, max_bin_edge, num_bins, N, fontsize, plot_hist, plot_norm,...
% plot_cumf, fig_hist, ax_hist, fig_norm, ax_norm, fig_cumf, ax_cumf, full_title )
%
% x_s               Vector of samples
% xmin              Minimum value of x for graph plotting
% xmax              Maximum value of x for graph plotting
% pdf_max           Maximum value of normalized histogram. If [] then auto-scale
% histogram_edges   Vector of histogram bin edges. If [] then compute using
%                   other inputs*
% f                 Vector of frequencies corresponding to histogram bins.
%                   If [] then compute from samples
% min_bin_edge*     Lower edge of histogram bins
% max_bin_edge*     Upper edge of histogram bins
% num_bins*         Number of histogram bins
% num_points        Number of data points for PDF overlay
% fontsize          Fontsize for graph annotation
% plot_hist         If == 1 then plot the histogram
% plot_norm         If == 1 then plot the normalized histogram
% plot_cumf         If == 1 then plot the cumulative frequency graph
% full_title        If == 1 then plot full information in graph titles,
%                   otherwise a more restricted set germane to the plot is
%                   presented
%
% fig_hist          Handle of histogram figure
% ax_hist           Handle of histogram axis
% fig_norm          Handle of normalized histogram figure
% ax_norm           Handle of normalized histogram axis
% fig_cumf          Handle of cumulatice frequency graph figure
% ax_cumf           Handle of cumulative frequency graph axis
%

function [ fig_hist, ax_hist, fig_norm, ax_norm, fig_cumf, ax_cumf, histogram_edges,f  ] = ...
    plot_histogram_and_cum_freq( x_s, xmin, xmax, pdf_max, histogram_edges, f,...
    min_bin_edge, max_bin_edge, num_bins, num_points, fontsize, plot_hist, plot_norm,...
    plot_cumf, fig_hist, ax_hist, fig_norm, ax_norm, fig_cumf, ax_cumf, full_title )

%% Perfrom statistical analysis %%
if isempty(f)
    % Analyse samples
    [x_mean, x_median, x_mode, x_std, x_skew,...
        cum_freq_exact, cum_freq, cum_freq_x, LQ, UQ, histogram_edges, histogram, f, norm_histogram,...
        pdf_fit, pdf_x ] = stats_from_samples( x_s, histogram_edges,...
        min_bin_edge, max_bin_edge, num_bins, num_points );
else
    % Analyse frequency table
    [x_mean, x_median, x_mode, x_std, x_skew,...
        cum_freq_exact, cum_freq, cum_freq_x, LQ,UQ, histogram_edges, histogram, norm_histogram,...
        pdf_fit, pdf_x ] = stats_from_freq_table( f, histogram_edges, num_points );
end

%

%% Plot histogram %%
if plot_hist==1
    
    %Point to desired figure and axes, or create new ones
    if isempty(fig_hist)
        fig_hist = figure( 'color',[1 1 1],'name','plot_histogram' );
    else
        figure(fig_hist);
    end
    if isempty(ax_hist)
        ax_hist = axes;
    else
        axes(ax_hist);
        cla;
    end

    %Plot histogram
    b = barchart( histogram_edges, histogram );
    hold on;
    
    %Plot coloured stars to indicate LQ, UQ median, mean etc
    LQ_marker = plot( LQ,0,'r*','markersize',fontsize   );
    median_marker = plot( x_median,0,'g*','markersize',fontsize   );
    mean_marker = plot( x_mean,0,'ko','markersize',fontsize   );
    UQ_marker = plot( UQ,0,'m*','markersize',fontsize   );
    
    %Set axes size and labels
    box on;
    axis tight
    xlabel('x','fontsize',fontsize);
    ylabel('frequency density','fontsize',fontsize)
    title({'Histogram',['Mean=',num2str(x_mean),', STD=',num2str(x_std),...
        ', SKEW=',num2str(x_skew),', Median=',num2str(x_median)],...
        ['LQ=',num2str(LQ),', UQ=',num2str(UQ),', IQR=',num2str(UQ-LQ)],...
        ['Number of samples = ',num2str(length(x_s))]},'fontsize',fontsize)
    grid on
    set(ax_hist,'fontsize',fontsize)
    legend([LQ_marker,median_marker,mean_marker,UQ_marker],...
        {'LQ','Median','Mean','UQ'},'location','eastoutside');
    
else
    fig_hist = [];
    ax_hist = [];
end

%

%% Plot normalized histogram %%
if plot_norm==1
    
    %Point to desired figure and axes, or create new ones
    if isempty(fig_norm)
        fig_norm = figure( 'color',[1 1 1],'name','plot_normalized_histogram' );
    else
        figure(fig_norm);
    end
    if isempty(ax_norm)
        ax_norm = axes;
    else
        axes(ax_norm);
        cla;
    end
    
    %Create histogram
    b = barchart( histogram_edges, norm_histogram );
    hold on;
    
    %Overlay PDF generated from parameters obtained from the data samples
    plot( pdf_x, pdf_fit );
    
    %Plot coloured stars to indicate LQ, UQ median, mean etc
    LQ_marker = plot( LQ,0,'r*','markersize',fontsize   );
    median_marker = plot( x_median,0,'g*','markersize',fontsize   );
    mean_marker = plot( x_mean,0,'ko','markersize',fontsize   );
    UQ_marker = plot( UQ,0,'m*','markersize',fontsize   );
    
    %Set axes size and labels
    box on;
    axis tight
    if isempty( pdf_max )
        ylimits = get( ax_norm, 'ylim' );
        pdf_max = ylimits(2);
    end
    xlim( [xmin,xmax] )
    ylim( [0, pdf_max ] );
    xlabel('x','fontsize',fontsize);
    ylabel('Probability density','fontsize',fontsize)
    if full_title ~=1
        %Less information
        title({'Normalized histogram with fitted PDF',...
            ['Mean=',num2str(x_mean),', Median=',num2str(x_median)],...
            ['STD=',num2str(x_std),', SKEW=',num2str(x_skew)]},'fontsize',fontsize)
    else
        %More information
        title( {'Normalized histogram with fitted PDF',...
            ['Mean=',num2str(x_mean),', Median=',num2str(x_median)],...
            ['STD=',num2str(x_std),', SKEW=',num2str(x_skew)],...
            ['LQ=',num2str(LQ),', UQ=',num2str(UQ),', IQR=',num2str(UQ-LQ)],...
            ['Number of samples = ',num2str(length(x_s))] },'fontsize',fontsize)
    end
    grid on
    set(ax_norm,'fontsize',fontsize)
    legend([LQ_marker,median_marker,mean_marker,UQ_marker],...
        {'LQ','Median','Mean','UQ'},'location','eastoutside');
else
    fig_norm =[];
    ax_norm = [];
end

%

%% Plot cumulative frequency graph %%
if plot_cumf==1
    
    %Point to desired figure and axes, or create new ones
    if isempty(fig_cumf)
        fig_cumf = figure( 'color',[1 1 1],'name','plot_cumulative_frequency_graph' );
    else
        figure(fig_cumf);
    end
    if isempty(ax_cumf)
        ax_cumf = axes;
    else
        axes(ax_cumf);
        cla;
    end
    
    %Generate cumulative frequency (%) plot
    plot( cum_freq_x,cum_freq,'b' );
    hold on;
    plot( histogram_edges, cum_freq_exact,'k+' );
    patch( [LQ,UQ,UQ,LQ],[0,0,75,75],'g','facealpha',0.2,'edgecolor','none');
    plot( [xmin,LQ], [25,25],'r' );
    plot( [LQ,LQ],[0,25] ,'r' );
    plot( [xmin,x_median], [50,50],'g' );
    plot( [x_median,x_median],[0,50] ,'g' );
    plot( [xmin,UQ], [75,75],'m' );
    plot( [UQ,UQ],[0,75] ,'m' );
    LQ_marker = plot( LQ,0,'r*','markersize',fontsize  );
    median_marker = plot( x_median,0,'g*','markersize',fontsize );
    mean_marker = plot( x_mean,0,'ko','markersize',fontsize  );
    UQ_marker = plot( UQ,0,'m*','markersize',fontsize  );
    xlabel('x','fontsize',14);
    ylabel('Cumulative frequency %','fontsize',fontsize)
    if full_title~=0
        %More information
        title({'Cumulative frequency %',['Mean=',num2str(x_mean),', STD=',num2str(x_std),...
            ', SKEW=',num2str(x_skew),', Median=',num2str(x_median)],...
            ['LQ=',num2str(LQ),', UQ=',num2str(UQ),', IQR=',num2str(UQ-LQ)],...
            ['Number of samples = ',num2str(length(x_s))]},'fontsize',fontsize)
    else
        %Less information
        title({'Cumulative frequency %',['Mean=',num2str(x_mean),', Median=',num2str(x_median)],...
            ['LQ=',num2str(LQ),', UQ=',num2str(UQ),', IQR=',num2str(UQ-LQ)] },'fontsize',fontsize)
    end
    grid on
    set(ax_cumf,'fontsize',fontsize)
    xlim( [xmin,xmax] )
    ylim([0,100])
    legend([LQ_marker,median_marker,mean_marker,UQ_marker],{'LQ','Median','Mean','UQ'},'location','eastoutside');
else
    fig_cumf = [];
    ax_cumf = [];
end

% End of code