%plot_skewed_normal
% Function which plots a skew-normal probability density function and
% shades this with a region corresponding to the range of a probability
% calculation P( x0 <= x < x1 )
%
% LAST UPDATED by Andy French July 2014
%
% Syntax: [fig,ax,P,pdf_max] = plot_skewed_normal( x_mean, x_std, x_skew, x0, x1, xmin, xmax,...
%    pdf_max, fontsize, N, ax, fig )
%
% x_mean        Mean of skew-normal distribution
% x_std         Standard deviation of skew-normal distribution
% x_skew        Skew of skew-normal distribution*
% x0            Lower range of probability calculation P( x0 <= x < x1 )
% x1            Upper range of probability calculation P( x0 <= x < x1 )
% xmin          Minimum x value for plot
% xmax          Maximum x value for plot
% pdf_max       Maximum value for PDF. If [] then use maximum of PDF.
% fontsize      Fontsize of plot annotation
% N             Number of data points for PDF computation.
% ax            Plot graph in a specified axes with handle ax. If [] then
%               axes will be create
% fig           Plot graph in a specified figure with handle fig. If []
%               then figure will be create
%
% P             Probability P( x0 <= x < x1 )
%
% * x_skew must be within range -1 < x_skew < 1. If it is outside this
% range, it will be internally fixed to range [-0.99,0.99]
%
% Example: 
% x_mean = 50; x_std = 20; x_skew = 0.6; x0 = 10; x1 = 60; xmin = 0; xmax = 100;
% pdf_max = []; fontsize = 14; N = 500; ax = []; fig = [];
% [fig,ax,P,pdf_max] = plot_skewed_normal( x_mean, x_std, x_skew, x0, x1, xmin, xmax,...
%    pdf_max, fontsize, N, ax, fig )

function [fig,ax,P,pdf_max] = plot_skewed_normal( x_mean, x_std, x_skew, x0, x1, xmin, xmax,...
    pdf_max, fontsize, N, ax, fig )

%Generate PDF
x = linspace( xmin, xmax, N );
[ x_s, pdf, cdf ] = skewed_normal( x_mean, x_std, x_skew, 1, x );

%Point to desired figure and axes, or create new ones
if isempty(fig)
    fig = figure( 'color',[1 1 1],'name','plot_skewed_normal' );
else
    figure(fig);
end
if isempty(ax)
    ax = axes;
else
    axes(ax);
    cla;
end

%Plot probability density function (PDF)
plot( x, pdf,'b','linewidth',2);

%Set grid and axes limits
hold on;
grid on;
axis tight
if isempty( pdf_max )
    ylimits = get( ax, 'ylim' );
    pdf_max = ylimits(2);
end
xlim( [xmin,xmax] )
ylim( [0, pdf_max ] );
set( ax,'fontsize',fontsize )
box on;

%Define patch for cumulative region
x0 = max( [x0, xmin ] );
x1 = min( [x1, xmax ] );
xx = linspace(x0,x1,N );
[ x_s, yy, cdf ] = skewed_normal( x_mean, x_std, x_skew, 1, xx );
patch( [x0,xx,x1,x0], [0,yy,0,0], 'g','facealpha',0.5);

%Plot mean
[ x_s, pdf_mean, cdf ] = skewed_normal( x_mean, x_std, x_skew, 1, x_mean );
plot( [x_mean,x_mean],[0,pdf_mean],'k--' );
plot( x_mean, 0, 'ko','markersize',fontsize );

%Compute probability corresponding to region x0 <= x < x1
[ x_s, pdf, cdf0 ] = skewed_normal( x_mean, x_std, x_skew, 1, x0 );
[ x_s, pdf, cdf1 ] = skewed_normal( x_mean, x_std, x_skew, 1, x1 );
P = cdf1 - cdf0;

%Set graph labelling
xlabel('x','fontsize',fontsize);
ylabel('Probability density','fontsize',fontsize)
title({'Skew-Normal PDF',['Mean=',num2str(x_mean),', STD=',num2str(x_std),...
    ', SKEW=',num2str(x_skew)],...
    ['P( ',num2str(x0),' \leq x < ',num2str(x1),' ) = ',num2str(P)] },'fontsize',fontsize)

%End of code