%skewed_normal
% The Skew Normal distribution is a generalization of the Gaussian (Normal)
% distribution. This function returns the probability density function,
% cumulative distribution function, and samples from the distribution.
%
% http://en.wikipedia.org/wiki/Skew_normal_distribution
%
% Syntax: [ x_s, pdf, cdf ] = skewed_normal( x_mean, x_std, x_skew, N, x )
%
% x_mean   Mean value of distribution E[x]
% x_std    Square root of the variance of the distribution V[x]
% x_skew   E[ ( (x - x_mean)/x_std )^3 ]
% N        Number of data samples
% x        Values of x at which the pdf and cdf are evaluated
%
% x_s      Vector of N samples from the distribution
% pdf      Probability density function of the distribution, evaluated at x
% cdf      Cumulative distribution function, evaluated at x
%
% NOTE: NaN values are returned for is abs( x_skew) > 1
%
% LAST UPDATED by Andy French July 2014

function [ x_s, pdf, cdf ] = skewed_normal( x_mean, x_std, x_skew, N, x )

%Check skew is in range  -1 < x_skew < 1
x_s = NaN(1,N);
pdf = NaN(1,N);
cdf = NaN(1,N);
if abs( x_skew) < 1
    
    if x_skew == 0
   
        %Use Normal ("Gaussian") distribution to determine exact forms of PDF and CDF
        w = x_std;
        ee = x_mean;
        zz = linspace( -10, 10, 500 );
        xx = w*zz + ee;
        [pdf,cdf] = gauss( x, x_mean, x_std );
        [ppdf,ccdf] = gauss( xx, x_mean, x_std );
        
    else
        
        %Numerically invert the skewer function using piecewise Hermite polynomials
        deltas = linspace(-0.99,0.99,500);
        skews = skewer(deltas);
        delta = interp1( skews, deltas, x_skew, 'spline' );
        
        %Determine other parameters of the Skew Normal PDF from the inputs
        w = x_std/sqrt( 1- (2/pi)*delta^2 );
        a = delta*sqrt( 1/(1-delta^2) );
        ee = x_mean - w*delta*sqrt(2/pi);
        
        %Determine pdf
        z = (x-ee)/w;
        pdf = (1/w)*(1/sqrt(2*pi))*exp( -0.5*z.^2 ).*( erf( a*z/sqrt(2) ) + 1 );
        
        %Determine cdf using a piecewise-polynomial fit numeric integration method
        zz = linspace( -10, 10, 500 );
        pp = (1/w)*(1/sqrt(2*pi))*exp( -0.5*zz.^2 ).*( erf( a*zz/sqrt(2) ) + 1 );
        cdf = w*integrate( zz,pp,-10, z );
        ccdf = w*integrate( zz,pp,-10, zz );
    end
    
    %Sample from the distribution ...
    
    %Determine N values of the CDF within the range [0,1]
    cdf_values = rand(1,N);
    
    %Interpolate cdf to find the corresponding x values
    [ccdf,i] = unique(ccdf);
    zz = zz(i);
    z_s = interp1( ccdf, zz, cdf_values, 'pchip' );
    x_s = w*z_s + ee;
end

%%

% Returns the skew (y) in terms of skew-normal distribution shape parameter d
function y = skewer(d)
y = d.^3;
y = y.*( -(2/pi)*d.^2 + 1 ).^(-3/2);
y = 0.5*(4-pi)*y*( (2/pi)^(3/2) );

%%

% Normal (Gaussian) distribution function
function [pdf,cdf] = gauss( x, x_mean, x_std )

%Proability density function (PDF)
pdf = (1/( x_std*sqrt(2*pi))) * exp( -( (x-x_mean).^2 )/( 2*x_std^2 ) );

%Cumulative distribution function
cdf = 0.5*( 1 + erf( (x-x_mean)/sqrt( 2*x_std^2 ) ) );

%%

% Test function - paste into command window to run
function skewed_normal_test

x = linspace( -10, 20, 300 );
x_mean = 2;
x_std = 4;
x_skew = -0.5;
N = 5000;
[ x_s, pdf, cdf ] = skewed_normal( x_mean, x_std, x_skew, N, x );

figure('color',[1 1 1],'name','Skew-Normal Distribution');

%PDF & normalized histogram
edges = linspace( min(x), max(x), 50 ) ;
spacing = edges(2)-edges(1);
h = histc(x_s,edges);
h = h/( spacing*sum(h) );
b = bar( edges, h, 'histc' );
set(b,'facecolor','g','facealpha',0.5)
hold on;
plot( x, pdf );
xlabel('x','fontsize',14);
ylabel('PDF','fontsize',14)
title({'Skew-Normal PDF',['Mean=',num2str(x_mean),', STD=',num2str(x_std),...
    ', SKEW=',num2str(x_skew)]},'fontsize',14)
grid on
set(gca,'fontsize',14)
print(gcf,'skew-normal PDF','-dpng','-r300')

%CDF
clf;
plot( x, cdf );
xlabel('x','fontsize',14);
ylabel('CDF','fontsize',14)
title('Skew-Normal CDF','fontsize',14)
grid on
set(gca,'fontsize',14)
print(gcf,'skew-normal CDF','-dpng','-r300')
close(gcf);

%End of code