%stats_from_excel_samples
% Loads an excel file containing a column of data samples and performs a
% statistical analysis. Averages, standard deviation and skew are computed
% and histograms and cumulative frequency + box & whisker overlays are also
% constructed. An Excel file containing a frequency table is also created.
%
% Optional input is a vector of histogram edges.
%
% LAST UPDATED by Andy French July 2014

function stats_from_excel_samples( varargin)

% Default number of histogram bins, PDF overlay points and fontsize
num_bins = 40;
num_points = 500;
fontsize = 14;

%

% Command line information
disp(' ')
disp(' STATISTICS FROM EXCEL SAMPLES ' )
disp(' by Andy French. July 2014' )
disp(' ')

% Choose Excel file to load. This must contain a single column of numeric data samples
[filename, pathname, filterindex] = uigetfile('*.xls', 'Choose an .xls file containing data samples');
if filename ==0
    return
end

%Start clock
t0 = tic;

%Read Excel file and obtain array of sample x_s
disp(['Performing analysis of file: ',filename])
[x_s,txt,raw] = xlsread( [pathname,'\',filename] );

%Set histogram bins
if nargin == 0
    min_bin_edge = min(x_s);
    max_bin_edge = max(x_s);
    num_bins = 40;
    histogram_edges = [];
else
    histogram_edges = varargin{1};
    min_bin_edge = min(histogram_edges);
    max_bin_edge = max(histogram_edges);
    num_bins = length(histogram_edges);
end

%Perform statistical analysis upon data samples
%Perform statistical analysis using stats_from_samples.m (called within
%plot_histogram_and_cum_freq.m) and plot graphs
pdf_max = [];
f = [];
plot_hist = 1;
plot_norm = 1;
plot_cumf = 1;
fig_hist=[];
ax_hist=[];
fig_norm=[];
ax_norm=[];
fig_cumf=[];
ax_cumf=[];
xmin = min(x_s);
xmax = max(x_s);
full_title = 1;
[ fig_hist, ax_hist, fig_norm, ax_norm, fig_cumf, ax_cumf, histogram_edges,f  ] = ...
    plot_histogram_and_cum_freq( x_s, xmin, xmax, pdf_max, histogram_edges,f,...
    min_bin_edge, max_bin_edge, num_bins, num_points, fontsize, plot_hist, plot_norm,...
    plot_cumf, fig_hist, ax_hist, fig_norm, ax_norm, fig_cumf, ax_cumf,...
    full_title );

%Create an output directory for graphs
output_dir = ['stats from xls samples ',strrep( datestr(now), ':','-' )];
mkdir(output_dir);

%Print graphs as PNG files to directory and close associated figures
print(fig_hist,[output_dir,'\samples histogram'],'-dpng','-r300')
close(fig_hist);
print(fig_norm,[output_dir,'\samples normalized histogram'],'-dpng','-r300')
close(fig_norm);
print(fig_cumf,[output_dir,'\samples cum_freq'],'-dpng','-r300')
close(fig_cumf);

%Write frequency table Excel file
freq_table{1,1} = 'x range';
freq_table{1,2} = 'frequency';
for n=1:length(histogram_edges)-1
    freq_table{1+n,1} = [ num2str(histogram_edges(n)),' <= x < ',num2str(histogram_edges(n+1))];
    freq_table{1+n,2} = f(n);
end
freq_table{1+length(histogram_edges),1} = ['x = ',num2str(histogram_edges(end)) ];
freq_table{1+length(histogram_edges),2} = f(length(histogram_edges));
xlswrite( [output_dir,'\skew-normal freq-table'], freq_table );

%Stop clock and display ending message
disp( ['Statistical analysis completed in ',num2str(toc(t0)),' seconds.'] )
disp(' ')

%End of code