%stats_from_freq_table
% Computes statistics from a frequency table of data. Also creates a
% histogram and cumulative frequency plot + box & whisker overlay.
%
% LAST UPADTED by Andy French July 2014

% Syntax: [x_mean, x_median, x_mode, x_std, x_skew,...
%    cum_freq, cum_freq_x, histogram_edges, histogram, norm_histogram,...
%    pdf_fit, pdf_x ] = stats_from_freq_table( f, histogram_edges, N )
%
% x_mean, x_median, x_mode     Data averages
% x_std, x_skew                Data standard deviation and skew
% cum_freq_exact               Cumulative frequency values (%) which correspond
%                              to histogram edges.
% cum_freq, cum_freq_x         Cumulative frequency (%) curve, with associated x values
% LQ                           Lower quartile x value (25% of cum. freq)
% UQ                           Upper quartile x value (75% of cum. freq)
% histogram_edges, histogram   Vector of histogram edges, and corresponding vector of frequencies.
% norm_histogram               Normalized histogram such that area is unity
% pdf_fit, pdf_x               Curve of PDF using skew-normal distribution
%
% f                            Vector of frequencies corresponding to
%                              x values in histogram_edges. Note f(end)
%                              corresponds to any samples that correspond to
%                              histogram_edges(end).
% histogram_edges              Vector of histogram bin edges. If [] then
%                              construct these.
% N                            Number of data values for PDF and cumulative
%                              frequency curve

function [x_mean, x_median, x_mode, x_std, x_skew,...
    cum_freq_exact, cum_freq, cum_freq_x, LQ,UQ, histogram_edges, histogram, norm_histogram,...
    pdf_fit, pdf_x ] = stats_from_freq_table( f, histogram_edges, N )

%Determine histogram bin coresponding to the highest frequency and hence
%estimate the modal value
nmax = mean( find( f == max(f) ) );
if nmax == length(histogram_edges)
    x_mode = histogram_edges(end);
else
    x_mode = 0.5*histogram_edges(nmax) + 0.5*histogram_edges(nmax+1);
end

%Determine the mid-bin values
x = [zeros(1,length(histogram_edges)-1 ),histogram_edges(end)];
for n=1:length(histogram_edges)-1
    x(n) = 0.5*( histogram_edges(n) + histogram_edges(n+1) );
end

%Estimate the mean of the grouped data
x_mean = sum( f.*x )/sum(f);

%Estimate the standard deviation and skew of the data
x_std = sqrt( sum( f.*x.^2 )/sum(f) - x_mean^2 );
x_skew = sum( f.*( (x-x_mean)/x_std ).^3 )/sum(f);

%Determine histogram (heigh being frequency density)
histogram = zeros(1,length(histogram_edges));
for n=1:length(histogram_edges)-1
    histogram(n) = f(n)/( histogram_edges(n+1) - histogram_edges(n) );
end
histogram(end) = f(end);

%Construct normalized histogram
norm_histogram = histogram;
h_area = 0;
for n=1:length(histogram_edges)-1
    h_area = h_area + histogram(n)*( histogram_edges(n+1) - histogram_edges(n) );
end
norm_histogram = norm_histogram/h_area;

%Construct cumulative frequency curve
cum_freq_exact = zeros(1,length(histogram_edges));
for n=2:length(histogram_edges)
    cum_freq_exact(n) = sum( f(1:n) );
end

%Turn into a %
cum_freq_exact = 100*cum_freq_exact/sum(f);

%Interpolate this curve
cum_freq_x = linspace( histogram_edges(1), histogram_edges(end), N );
cum_freq = interp1( histogram_edges, cum_freq_exact, cum_freq_x, 'spline' );

%Determine LQ, median and UQ
LQ = interp1( cum_freq, cum_freq_x, 25, 'spline' );
x_median = interp1( cum_freq, cum_freq_x, 50, 'spline' );
UQ = interp1( cum_freq, cum_freq_x, 75, 'spline' );

%Determine Probability Density Function
pdf_x = cum_freq_x;
[ x_s, pdf_fit, cdf ] = skewed_normal( x_mean, x_std, x_skew, [], pdf_x );

%End of code