%stats_from_samples
% Computes statistics from a vector of data samples. Also creates a
% histogram and cumulative frequency plot + box & whisker overlay.
%
% LAST UPADTED by Andy French July 2014
%
% Syntax: [x_mean, x_median, x_mode, x_std, x_skew,...
%    cum_freq_exact, cum_freq, cum_freq_x, LQ,UQ, histogram_edges, histogram, f, norm_histogram,...
%    pdf_fit, pdf_x ] = stats_from_samples( x, histogram_edges,...
%    min_bin_edge, max_bin_edge, num_bins, N )
%
% x_mean, x_median, x_mode     Data averages
% x_std, x_skew                Data standard deviation and skew
% cum_freq_exact               Cumulative frequency values (%) which correspond
%                              to histogram edges.
% cum_freq, cum_freq_x         Cumulative frequency (%) curve, with
%                              associated x values (N values between max and min of histogram)
% LQ                           Lower quartile x value (25% of cum. freq)
% UQ                           Upper quartile x value (75% of cum. freq)
% histogram_edges, histogram   Vector of histogram edges, and corresponding
%                              vector of frequency density (histogram) and frequency (f)
% norm_histogram               Normalized histogram such that area is unity
% pdf_fit, pdf_x               Curve of PDF using skew-normal distribution
%
% x                            Vector of data samples
% histogram_edges              Vector of histogram bin edges. If [] then
%                              construct these.
% min_bin_edge, max_bin_edge   If isempty(histogram_edges) then specify
%                              maximum and minimum histogram bin edges
% num_bins                     If isempty(histogram_edges then specify
%                              number of frequency bins between min and max bin edges
% N                            Number of data values for PDF and cumulative
%                              frequency curve

function [x_mean, x_median, x_mode, x_std, x_skew,...
    cum_freq_exact, cum_freq, cum_freq_x, LQ,UQ, histogram_edges, histogram, f, norm_histogram,...
    pdf_fit, pdf_x ] = stats_from_samples( x, histogram_edges,...
    min_bin_edge, max_bin_edge, num_bins, N )

%Determine averages
x_mean = mean(x);
x_median = median(x);
x_mode = mode(x);

%Determine standard deviation and skew of samples
x_std = std(x);
x_skew = mean( ( (x-x_mean)/x_std ).^3 );

%Construct edges vector for histogram
if isempty( histogram_edges )
    histogram_edges = linspace( min_bin_edge, max_bin_edge, num_bins );
else
    num_bins = length( histogram_edges );
end

%Construct histogram
f = histc( x, histogram_edges );

%Scale such that height of histogram is actually frequency density and area
%is frequency
histogram = f;
for n=1:num_bins-1
    histogram(n) = f(n)/( histogram_edges(n+1) - histogram_edges(n) );
end
histogram(end) = 0; %Ignore samples which equal the maximum bin edge.
f(end) = 0;

%Construct normalized histogram
norm_histogram = histogram;
h_area = 0;
for n=1:num_bins-1
    h_area = h_area + histogram(n)*( histogram_edges(n+1) - histogram_edges(n) );
end
norm_histogram = norm_histogram/h_area;

%Construct cumulative frequency curve
cum_freq_exact = zeros(1,num_bins);
for n=2:num_bins
    cum_freq_exact(n) = length( find( x <= histogram_edges(n) ) );
end

%Turn into a %
cum_freq_exact = 100*cum_freq_exact/length(x);

%Interpolate this curve
cum_freq_x = linspace( min_bin_edge, max_bin_edge, N );
cum_freq = interp1( histogram_edges, cum_freq_exact, cum_freq_x, 'spline' );

%Determine LQ and UQ
LQ = interp1( cum_freq, cum_freq_x, 25, 'spline' );
UQ = interp1( cum_freq, cum_freq_x, 75, 'spline' );

%Determine Probability Density Function
pdf_x = cum_freq_x;
[ x_s, pdf_fit, cdf ] = skewed_normal( x_mean, x_std, x_skew, N, pdf_x );

%End of code